from typing import Optional

from truss.remote.baseten.api import BasetenApi
from truss.remote.baseten.auth import AuthService
from truss.remote.baseten.core import (
    archive_truss,
    create_model,
    exists_model,
    upload_model,
)
from truss.remote.baseten.service import BasetenService
from truss.remote.baseten.utils.transfer import base64_encoded_json_str
from truss.remote.truss_remote import TrussRemote
from truss.truss_handle import TrussHandle


class BasetenRemote(TrussRemote):
    def __init__(self, remote_url: str, api_key: Optional[str] = None, **kwargs):
        super().__init__(remote_url, **kwargs)
        if api_key:
            self._auth_service = AuthService(api_key=api_key)
        else:
            self._auth_service = AuthService()

    def authenticate(self):
        return self._auth_service.validate()

    def push(self, truss_handle: TrussHandle, model_name: str):  # type: ignore
        self.authenticate()
        api = BasetenApi(f"{self._remote_url}/graphql/", self._auth_service)

        if model_name.isspace():
            raise ValueError("Model name cannot be empty")

        if exists_model(api, model_name):
            raise ValueError(f"Model with name {model_name} already exists")

        gathered_truss = TrussHandle(truss_handle.gather())
        encoded_config_str = base64_encoded_json_str(
            gathered_truss._spec._config.to_dict()
        )

        temp_file = archive_truss(gathered_truss)
        s3_key = upload_model(api, temp_file)
        model_id, model_version_id = create_model(
            api=api,
            model_name=model_name,
            s3_key=s3_key,
            config=encoded_config_str,
        )

        return BasetenService(
            model_id=model_id,
            model_version_id=model_version_id,
            is_draft=False,
            api_key=self._auth_service.authenticate().value,
            service_url=f"{self._remote_url}/model_versions/{model_version_id}",
            truss_handle=truss_handle,
        )



import logging
import os
from typing import Optional

from truss.remote.baseten.error import AuthorizationError

logger = logging.getLogger(__name__)


class ApiKey:
    value: str

    def __init__(self, value: str):
        self.value = value

    def header(self):
        return {"Authorization": f"Api-Key {self.value}"}


class AuthService:
    def __init__(self, api_key: Optional[str] = None):
        if api_key is not None:
            self.set_key(api_key)

    def validate(self):
        if "BASETEN_API_KEY" not in os.environ:
            raise AuthorizationError(
                "Could not find BASETEN_API_KEY in environment variables."
            )

    def authenticate(self) -> ApiKey:
        self.validate()
        return ApiKey(os.environ["BASETEN_API_KEY"])

    def set_key(self, api_key: str) -> ApiKey:
        logger.warning(
            "Setting BASETEN_API_KEY in environment variables. This may not persist."
        )
        os.environ["BASETEN_API_KEY"] = api_key
        return self.authenticate()



#!/usr/bin/env python3
import logging
import subprocess
import sys
from pathlib import Path
from typing import Any, List, Optional

from truss.contexts.image_builder.serving_image_builder import ServingImageBuilder
from truss.contexts.local_loader.docker_build_emulator import DockerBuildEmulator
from truss.contexts.local_loader.truss_file_syncer import TrussFilesSyncer
from truss.patch.local_truss_patch_applier import LocalTrussPatchApplier
from truss.util.path import build_truss_shadow_target_directory


def create_venv_builder():
    import venv

    class VenvBuilder(venv.EnvBuilder):
        """Virtual Environment Builder

        This class handles setting up a virtual environment in `venv_dir`.
        """

        def __init__(self, venv_dir: Path, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.context: Any = None
            self.venv_dir = venv_dir

        def post_setup(self, context):
            """Implement base class `post_setup` hook to store venv context."""
            self.context = context

        def create_with_requirements(self, req_files: List[str]) -> None:
            """Create virtualenv with the requirements specified in the files provided.

            Args:
                `req_file` (List[str]): List of requirement files as subpaths to `self.venv_dir`.
            """
            from yaspin import yaspin

            with yaspin(text="Creating virtual environment") as spinner:
                env_dir = str(self.venv_dir / ".env")
                super().create(env_dir)
                spinner.ok("✅")
            with yaspin(text="Installing depedencies") as spinner:
                self._upgrade_pip()
                for req_file in req_files:
                    self._install_pip_requirements(self.venv_dir / req_file)
                spinner.ok("✅")

        def _upgrade_pip(self):
            """Helper function to upgrade pip."""
            subprocess.check_call(
                [
                    self.context.env_exe,
                    "-m",
                    "pip",
                    "install",
                    "--upgrade",
                    "pip",
                ],
                stdout=subprocess.PIPE,
            )

        def _install_pip_requirements(self, req_file: Path):
            """Helper function to install requirements from file."""
            if req_file.exists():
                pip_install_command = [
                    self.context.env_exe,
                    "-m",
                    "pip",
                    "install",
                    "-r",
                    str(req_file.absolute()),
                ]
                subprocess.check_call(
                    pip_install_command,
                    stdout=subprocess.PIPE,
                )

    return VenvBuilder


class LocalServerLoader:
    """Handle the setup and loading of truss server locally."""

    def __init__(
        self,
        truss_path: Path,
        context_builder: ServingImageBuilder,
        port: Optional[int] = None,
    ) -> None:
        self.truss_path = truss_path
        self.context_builder = context_builder
        self.port = port or 8080

    def watch(
        self,
        build_dir: Optional[Path] = None,
        venv_dir: Optional[Path] = None,
    ):
        """Run the server and watch for changes"""
        from yaspin import yaspin

        def _prep_or_create_dir(location: Optional[Path], stub: str) -> Path:
            if location is None:
                location = build_truss_shadow_target_directory(stub, self.truss_path)
            else:
                if not location.exists():
                    location.mkdir(parents=True)
            return location

        build_dir = _prep_or_create_dir(build_dir, "build")
        venv_dir = _prep_or_create_dir(venv_dir, "server_venv")

        with yaspin(text="Preparing truss context") as spinner:
            self.context_builder.prepare_image_build_dir(build_dir)
            dockerfile_path = build_dir / "Dockerfile"
            docker_build_emulator = DockerBuildEmulator(dockerfile_path, build_dir)
            build_result = docker_build_emulator.run(venv_dir)
            spinner.ok("✅")

        VenvBuilder = create_venv_builder()
        venv_builder = VenvBuilder(venv_dir, with_pip=True)
        requirements_files = [
            "app/requirements.txt",
            "requirements.txt",
        ]
        venv_builder.create_with_requirements(requirements_files)

        TrussFilesSyncer(
            self.truss_path,
            LocalTrussPatchApplier(
                venv_dir / "app/",
                venv_builder.context.env_exe,
                logging.Logger(__name__),
            ),
        ).start()

        subprocess.check_call(
            [
                venv_builder.context.env_exe,
                "-m",
                "uvicorn",
                "local_inference_server:app",
                "--reload",
                "--port",
                str(self.port),
            ],
            cwd=str(venv_dir / str(build_result.workdir).replace("/", "", 1)),
            env={
                **build_result.env,
                "SETUP_JSON_LOGGER": "False",
            },
            stdin=sys.stdin,
            stdout=sys.stdout,
            stderr=sys.stderr,
        )



from typing import Dict, Optional

from truss.remote.baseten.auth import AuthService
from truss.remote.truss_remote import TrussService
from truss.truss_handle import TrussHandle


class BasetenService(TrussService):
    def __init__(
        self,
        model_id: str,
        model_version_id: str,
        is_draft: bool,
        api_key: str,
        service_url: str,
        truss_handle: Optional[TrussHandle] = None,
    ):

        super().__init__(is_draft=is_draft, service_url=service_url)
        self._model_id = model_id
        self._model_version_id = model_version_id
        self._auth_service = AuthService(api_key=api_key)
        self._truss_handle = truss_handle

    @property
    def is_live(self) -> bool:
        return False

    @property
    def is_ready(self) -> bool:
        return False

    def predict(self, model_request_body: Dict):
        invocation_url = f"{self._service_url}/predict"
        response = self._send_request(invocation_url, "POST", data=model_request_body)
        return response.json()["model_output"]

    def authenticate(self) -> dict:
        return self._auth_service.authenticate().header()



import json
import logging
import os
from functools import wraps
from pathlib import Path
from typing import Callable, List, Optional, Union

import click
import truss
import yaml
from truss.remote.baseten import BasetenRemote

logging.basicConfig(level=logging.INFO)


def echo_output(f: Callable[..., object]):
    @wraps(f)
    def wrapper(*args, **kwargs):
        click.echo(f(*args, **kwargs))

    return wrapper


def error_handling(f: Callable[..., object]):
    @wraps(f)
    def wrapper(*args, **kwargs):
        try:
            f(*args, **kwargs)
        except ValueError as e:
            click.echo(e)
            print_help()
        except Exception as e:
            click.echo(e)

    return wrapper


def print_help() -> None:
    ctx = click.get_current_context()
    click.echo(ctx.get_help())


@click.group(name="truss", invoke_without_command=True)
@click.pass_context
@click.option(
    "-v",
    "--version",
    is_flag=True,
    show_default=False,
    default=False,
    help="Show Truss package version.",
)
def cli_group(ctx, version) -> None:
    if not ctx.invoked_subcommand:
        if version:
            click.echo(truss.version())
        else:
            click.echo(ctx.get_help())


@cli_group.command()
@click.argument("target_directory", required=True)
@click.option(
    "-s",
    "--skip-confirm",
    is_flag=True,
    show_default=True,
    default=False,
    help="Skip confirmation prompt.",
)
@click.option(
    "-t",
    "--trainable",
    is_flag=True,
    show_default=True,
    default=False,
    help="Create a trainable truss.",
)
@error_handling
def init(target_directory, skip_confirm, trainable) -> None:
    """Initializes an empty Truss directory.

    TARGET_DIRECTORY: A Truss is created in this directory
    """
    tr_path = Path(target_directory)
    if skip_confirm or click.confirm(f"A Truss will be created at {tr_path}"):
        truss.init(target_directory=target_directory, trainable=trainable)
        click.echo(f"Truss was created in {tr_path}")


@cli_group.command()
@click.argument("build_dir")
@click.argument("target_directory", required=False)
@error_handling
def build_context(build_dir, target_directory: str) -> None:
    """
    Create a docker build context for a Truss.

    BUILD_DIR: Folder where image context is built for Truss

    TARGET_DIRECTORY: A Truss directory. If none, use current directory.
    """
    tr = _get_truss_from_directory(target_directory=target_directory)
    tr.docker_build_setup(build_dir=Path(build_dir))


@cli_group.command()
@click.argument("target_directory", required=False)
@click.argument("build_dir", required=False)
@error_handling
@click.option("--tag", help="Docker image tag")
def build_image(target_directory: str, build_dir: Path, tag) -> None:
    """
    Builds the docker image for a Truss.

    TARGET_DIRECTORY: A Truss directory. If none, use current directory.

    BUILD_DIR: Image context. If none, a temp directory is created.
    """
    tr = _get_truss_from_directory(target_directory=target_directory)
    if build_dir:
        build_dir = Path(build_dir)
    tr.build_serving_docker_image(build_dir=build_dir, tag=tag)


@cli_group.command()
@click.argument("target_directory", required=False)
@click.argument("build_dir", required=False)
@click.option("--tag", help="Docker build image tag")
@click.option("--port", type=int, default=8080, help="Local port used to run image")
@click.option(
    "--attach", is_flag=True, default=False, help="Flag for attaching the process"
)
@error_handling
def run_image(target_directory: str, build_dir: Path, tag, port, attach) -> None:
    """
    Runs the docker image for a Truss.

    TARGET_DIRECTORY: A Truss directory. If none, use current directory.

    BUILD_DIR: Image context. If none, a temp directory is created.
    """
    tr = _get_truss_from_directory(target_directory=target_directory)
    if build_dir:
        build_dir = Path(build_dir)
    urls = tr.get_urls_from_truss()
    if urls:
        click.confirm(
            f"Container already exists at {urls}. Are you sure you want to continue?"
        )
    tr.docker_run(build_dir=build_dir, tag=tag, local_port=port, detach=not attach)


@cli_group.command()
@click.argument("target_directory", required=False)
@click.option("--build-dir", type=Path, required=False)
@click.option("--venv-dir", type=Path, required=False)
@click.option("--port", type=int, default=8080, help="Local port used to run server")
@error_handling
def watch(
    target_directory: str,
    build_dir: Optional[Path],
    venv_dir: Optional[Path],
    port: int,
) -> None:
    """
    Runs the model server for a Truss.

    TARGET_DIRECTORY: A Truss directory. If none, use current directory.

    BUILD_DIR: Image context. If none, a temp directory is created.
    """
    if build_dir:
        build_dir = Path(build_dir)

    if venv_dir:
        venv_dir = Path(venv_dir)
    tr = _get_truss_from_directory(target_directory=target_directory)
    tr.run_local_server_with_reload(build_dir=build_dir, work_dir=venv_dir, port=port)


@cli_group.command()
@click.option("--target_directory", required=False, help="Directory of truss")
@click.option(
    "--request",
    type=str,
    required=False,
    help="String formatted as json that represents request",
)
@click.option(
    "--build-dir",
    type=click.Path(exists=True),
    required=False,
    help="Directory where context is built",
)
@click.option("--tag", help="Docker build image tag")
@click.option("--port", type=int, default=8080, help="Local port used to run image")
@click.option(
    "--no-docker",
    is_flag=True,
    default=False,
    help="Flag to run prediction with a docker container",
)
@click.option(
    "--request-file",
    type=click.Path(exists=True),
    help="Path to json file containing the request",
)
@error_handling
@echo_output
def predict(
    target_directory: str,
    request: Union[bytes, str],
    build_dir,
    tag,
    port,
    no_docker,
    request_file,
):
    """
    Invokes the packaged model, either locally or in a Docker container.

    TARGET_DIRECTORY: A Truss directory. If none, use current directory.

    REQUEST: String formatted as json that represents request

    BUILD_DIR: Directory where context is built. If none, a temp directory is created.

    TAG: Docker build image tag

    PORT: Local port used to run image

    NO_DOCKER: Flag to run prediction without a docker container

    REQUEST_FILE: Path to json file containing the request
    """
    if request is not None:
        request_data = json.loads(request)
    elif request_file is not None:
        with open(request_file) as json_file:
            request_data = json.load(json_file)
    else:
        raise ValueError("At least one of request or request-file must be supplied.")

    tr = _get_truss_from_directory(target_directory=target_directory)
    if no_docker:
        return tr.server_predict(request_data)
    else:
        return tr.docker_predict(
            request_data, build_dir=build_dir, tag=tag, local_port=port, detach=True
        )


@cli_group.command()
@click.option("--target_directory", required=False, help="Directory of truss")
@click.option(
    "--build-dir",
    type=click.Path(exists=True),
    required=False,
    help="Directory where context is built",
)
@click.option("--tag", help="Docker build image tag")
@click.option(
    "--var",
    multiple=True,
    help="""Training variables in key=value form where value is string.
    For more complex values use vars_yaml_file""",
)
@click.option(
    "--vars_yaml_file",
    required=False,
    help="Training variables from a yaml file",
)
@click.option(
    "--local",
    is_flag=True,
    default=False,
    help="Flag to run training locally (not on docker)",
)
@error_handling
@echo_output
def train(target_directory: str, build_dir, tag, var: List[str], vars_yaml_file, local):
    """Runs prediction for a Truss in a docker image or locally"""
    tr = _get_truss_from_directory(target_directory=target_directory)
    if vars_yaml_file is not None:
        with Path(vars_yaml_file).open() as vars_file:
            variables = yaml.safe_load(vars_file)
    else:
        variables = _variables_dict_from_option(var)
    if local:
        return tr.local_train(variables=variables)

    return tr.docker_train(build_dir=build_dir, tag=tag, variables=variables)


@cli_group.command()
@click.argument("target_directory", required=False, default=os.getcwd())
@click.option("--api-key", type=str, required=False, help="Your API key")
@click.option("--model-name", type=str, required=True, help="Name of the model")
@click.option(
    "--remote-name",
    type=str,
    required=False,
    help="Name of the remote",
    default="baseten",
)
@click.option("--remote-url", type=str, required=False, help="URL of the remote")
@error_handling
def push(
    target_directory: str,
    api_key: str,
    model_name: str,
    remote_name: str,
    remote_url: str,
) -> None:
    """
    Pushes a truss to a TrussRemote.

    TARGET_DIRECTORY: A Truss directory. If none, use current directory.

    """
    registry = {"baseten": (BasetenRemote, "https://app.baseten.co")}

    if remote_name not in registry:
        raise ValueError(
            f"Remote {remote_name} not found. Available remotes: {list(registry.keys())}"
        )

    if not api_key:
        api_key = os.environ.get(f"{remote_name.upper()}_API_KEY", None)  # type: ignore
        if not api_key:
            raise ValueError(
                f"API key must be provided either as a flag "
                f"or an environment variable"
                f"({remote_name.upper()}_API_KEY)"
            )

    tr = _get_truss_from_directory(target_directory=target_directory)

    # Instantiate remote
    remote_cls = registry[remote_name][0]
    remote_url = remote_url or registry[remote_name][1]
    remote = remote_cls(remote_url, api_key)

    # Push
    service = remote.push(tr, model_name)

    click.echo(f"Model {model_name} was successfully pushed.")
    click.echo(f"Service URL: {service._service_url}")


@cli_group.command()
@click.argument("target_directory", required=False)
@click.option("--name", type=str, required=False, help="Name of example to run")
@click.option(
    "--local", is_flag=True, default=False, help="Flag to run prediction locally"
)
@error_handling
@echo_output
def run_example(target_directory: str, name, local):
    """
    Runs examples specified in the Truss, over docker.

    TARGET_DIRECTORY: A Truss directory. If none, use current directory.
    """
    tr = _get_truss_from_directory(target_directory=target_directory)
    predict_fn = tr.docker_predict
    if local:
        predict_fn = tr.server_predict

    if name is not None:
        example = tr.example(name)
        click.echo(f"Running example: {name}")
        return predict_fn(example.input)
    else:
        example_outputs = []
        for example in tr.examples():
            click.echo(f"Running example: {example.name}")
            example_outputs.append(predict_fn(example.input))
        return example_outputs


@cli_group.command()
@click.argument("target_directory", required=False)
@error_handling
def get_container_logs(target_directory) -> None:
    """
    Get logs in a container is running for a truss

    TARGET_DIRECTORY: A Truss directory. If none, use current directory.
    """
    for log in _get_truss_from_directory(
        target_directory=target_directory
    ).serving_container_logs():
        click.echo(log)


@cli_group.command()
@click.argument("target_directory", required=False)
def kill(target_directory: str) -> None:
    """
    Kills containers related to truss.

    TARGET_DIRECTORY: A Truss directory. If none, use current directory.
    """
    tr = _get_truss_from_directory(target_directory=target_directory)
    tr.kill_container()


@cli_group.command()
def kill_all() -> None:
    "Kills all truss containers that are not manually persisted"
    truss.kill_all()


@cli_group.command()
@error_handling
def cleanup() -> None:
    """
    Clean up truss data.

    Truss creates temporary directories for various operations
    such as for building docker images. This command clears
    that data to free up disk space.
    """
    truss.build.cleanup()


def _get_truss_from_directory(target_directory: Optional[str] = None):
    """Gets Truss from directory. If none, use the current directory"""
    if target_directory is None:
        target_directory = os.getcwd()
    return truss.load(target_directory)


def _variables_dict_from_option(variables_list: List[str]) -> dict:
    vars_dict = {}
    for var in variables_list:
        first_equals_pos = var.find("=")
        if first_equals_pos == -1:
            raise ValueError(
                f"Training variable expected in `key=value` from but found `{var}`",
            )
        var_name = var[:first_equals_pos]
        var_value = var[first_equals_pos + 1 :]
        vars_dict[var_name] = var_value
    return vars_dict


if __name__ == "__main__":
    cli_group()



import logging

import requests
from truss.remote.baseten.auth import AuthService
from truss.remote.baseten.error import ApiError

logger = logging.getLogger(__name__)


class BasetenApi:
    """
    A client for the Baseten API.

    Args:
        api_url: The URL of the Baseten API.
        auth_service: An AuthService instance.
    """

    def __init__(self, api_url: str, auth_service: AuthService):
        self._api_url = api_url
        self._auth_service = auth_service
        self._auth_token = self._auth_service.authenticate()

    def _post_graphql_query(self, query_string: str) -> dict:
        headers = self._auth_token.header()
        resp = requests.post(
            self._api_url,
            data={"query": query_string},
            headers=headers,
            timeout=120,
        )

        if not resp.ok:
            logger.error(f"GraphQL endpoint failed with error: {resp.content}")  # type: ignore
            resp.raise_for_status()

        resp_dict = resp.json()
        errors = resp_dict.get("errors")
        if errors:
            raise ApiError(errors[0]["message"], resp)
        return resp_dict

    def model_s3_upload_credentials(self):
        query_string = """
        {
            model_s3_upload_credentials {
                s3_bucket
                s3_key
                aws_access_key_id
                aws_secret_access_key
                aws_session_token
            }
        }
        """
        resp = self._post_graphql_query(query_string)
        return resp["data"]["model_s3_upload_credentials"]

    def create_model_from_truss(
        self,
        model_name,
        s3_key,
        config,
        semver_bump,
        client_version,
        is_trusted=False,
    ):
        query_string = f"""
        mutation {{
        create_model_from_truss(name: "{model_name}",
                    s3_key: "{s3_key}",
                    config: "{config}",
                    semver_bump: "{semver_bump}",
                    client_version: "{client_version}",
                    is_trusted: {'true' if is_trusted else 'false'}
    ) {{
            id,
            name,
            version_id
        }}
        }}
        """
        resp = self._post_graphql_query(query_string)
        return resp["data"]["create_model_from_truss"]

    def models(self):
        query_string = """
        {
            models {
                id,
                name
                versions{
                    id,
                    semver,
                    current_deployment_status,
                    is_primary,
                }
            }
        }
        """

        resp = self._post_graphql_query(query_string)
        return resp["data"]



import logging
from typing import IO, Optional, Tuple

import truss
from truss.remote.baseten.api import BasetenApi
from truss.remote.baseten.utils.tar import create_tar_with_progress_bar
from truss.remote.baseten.utils.transfer import multipart_upload_boto3
from truss.truss_handle import TrussHandle

logger = logging.getLogger(__name__)


def exists_model(api: BasetenApi, model_name: str) -> bool:
    """
    Check if a model with the given name exists in the Baseten remote.

    Args:
        api: BasetenApi instance
        model_name: Name of the model to check for existence

    Returns:
        True if the model exists, False otherwise
    """
    models = api.models()
    for model in models["models"]:
        if model["name"] == model_name:
            return True
    return False


def archive_truss(b10_truss: TrussHandle) -> IO:
    """
    Archive a TrussHandle into a tar file.

    Args:
        b10_truss: TrussHandle to archive

    Returns:
        A file-like object containing the tar file
    """
    try:
        truss_dir = b10_truss._spec.truss_dir
        temp_file = create_tar_with_progress_bar(truss_dir)
    except PermissionError:
        # Windows bug with Tempfile causes PermissionErrors
        temp_file = create_tar_with_progress_bar(truss_dir, delete=False)
    temp_file.file.seek(0)
    return temp_file


def upload_model(api: BasetenApi, serialize_file: IO) -> str:
    """
    Upload a TrussHandle to the Baseten remote.

    Args:
        api: BasetenApi instance
        serialize_file: File-like object containing the serialized TrussHandle

    Returns:
        The S3 key of the uploaded file
    """
    temp_credentials_s3_upload = api.model_s3_upload_credentials()
    s3_key = temp_credentials_s3_upload.pop("s3_key")
    s3_bucket = temp_credentials_s3_upload.pop("s3_bucket")
    multipart_upload_boto3(
        serialize_file.name, s3_bucket, s3_key, temp_credentials_s3_upload
    )
    return s3_key


def create_model(
    api: BasetenApi,
    model_name: str,
    s3_key: str,
    config: str,
    semver_bump: Optional[str] = "MINOR",
    is_trusted: Optional[bool] = False,
) -> Tuple[str, str]:
    """
    Create a model in the Baseten remote.

    Args:
        api: BasetenApi instance
        model_name: Name of the model to create
        s3_key: S3 key of the uploaded TrussHandle
        config: Base64 encoded JSON string of the Truss config
        semver_bump: Semver bump type, defaults to "MINOR"
        is_trusted: Whether the model is trusted, defaults to False

    Returns:
        A tuple of the model ID and version ID
    """
    model_version_json = api.create_model_from_truss(
        model_name,
        s3_key,
        config,
        semver_bump,
        f"truss=={truss.version()}",
        is_trusted,
    )

    return (model_version_json["id"], model_version_json["version_id"])



import json


class Error(Exception):
    """Base Baseten Error"""

    def __init__(self, message):
        super().__init__(message)
        self.message = message


class ApiError(Error):
    """Errors in calling the Baseten API."""

    def __init__(self, message, response=None):
        super().__init__(message)
        self.response = response

    def __str__(self):
        error_str = self.message
        if (
            self.response is not None
        ):  # non-200 Response objects are falsy, hence the not None.
            error_message = json.loads(self.response.content)
            error_message = (
                error_message["error"] if "error" in error_message else error_message
            )
            error_str = f"{error_str}\n<Server response: {error_message}>"
        return error_str


class AuthorizationError(Error):
    """Raised in places where the user needs to be logged in and is not."""

    pass
